// stkbrass.cpp
//
// A simple brass FM implementation as described by Chowning, AES, 1973.
//
// by Gary P. Scavone, McGill University, 2005.
//
// Assuming the stk directory is on the Desktop (with compiled stk library in src/ sub-directory)
// and stkbrass.cpp file in the current directory, you can compile this with:
//    g++ -std=c++11 -I/Users/me/Desktop/stk/include/ -L/Users/me/Desktop/stk/src/ -D__MACOSX_CORE__ stkbrass.cpp -lstk -lpthread -framework CoreAudio -framework CoreMIDI -framework CoreFoundation

#include "ADSR.h"
#include "SineWave.h"
#include "RtWvOut.h"

using namespace stk;

// WARNING: no error checking here!

int main()
{
  // Set the global sample rate before creating class instances.
  Stk::setSampleRate( 44100.0 );

  ADSR adsr;

  // Create looping wavetables with sine waveforms.
  SineWave carrier;
  SineWave modulator;

  // Create a one-channel realtime output object.
  RtWvOut output;
  output.start();

  // Set our FM parameters.
  float f = 440.0;
  modulator.setFrequency( f );
  adsr.setAllTimes( 0.1, 0.1, 0.4, 0.1 );
  adsr.keyOn();
  float maxD = 5.0 * f;
  long releaseCount = (long) (Stk::sampleRate() * 0.5);

  // Start the runtime loop.
  long counter = 0;
  while ( adsr.getState() != ADSR::IDLE ) {

    // Do FM calculations.
    carrier.setFrequency( adsr.lastOut() * maxD * modulator.tick() + f );
    output.tick( adsr.lastOut() * carrier.tick() );

    // Update the envelope and check for release time.
    adsr.tick();
    if ( counter++ == releaseCount )
      adsr.keyOff();
  }

  return 0;
}
